//!
//!
use super::{models, schema::member_levels};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of member_levels").finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<models::MemberLevel, Error> {
    member_levels::table
      .find(id as uuid::Uuid)
      .first::<models::MemberLevel>(conn)
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = member_levels::table.into_boxed();

    if let Some(sp) = find_all.name_like {
      q = q.filter(member_levels::dsl::name.like(sp));
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: models::FindAll,
  ) -> Result<Vec<models::MemberLevel>, Error> {
    let mut q = member_levels::table.into_boxed();

    if let Some(sp) = find_all.name_like {
      q = q.filter(member_levels::dsl::name.like(sp));
    }

    if let Some(p) = find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = find_all.sorts {
      for s in orderbys {
        match s {
          bcr::models::pagination::Sort::ASC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(member_levels::name.asc());
            }
            "sort_order" => {
              q = q.order_by(member_levels::sort_order.asc());
            }
            "created_at" => {
              q = q.order_by(member_levels::created_at.asc());
            }
            "updated_at" => {
              q = q.order_by(member_levels::updated_at.asc());
            }
            "deleted_at" => {
              q = q.order_by(member_levels::deleted_at.asc());
            }
            _ => {}
          },
          bcr::models::pagination::Sort::DESC(property) => match property.as_str() {
            "name" => {
              q = q.order_by(member_levels::name.desc());
            }
            "sort_order" => {
              q = q.order_by(member_levels::sort_order.desc());
            }
            "created_at" => {
              q = q.order_by(member_levels::created_at.desc());
            }
            "updated_at" => {
              q = q.order_by(member_levels::updated_at.desc());
            }
            "deleted_at" => {
              q = q.order_by(member_levels::deleted_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::MemberLevel>(conn)
  }
}
